﻿<#
    .SYNOPSIS
       Change the IP address of a VM without logging on to the VM.
    .DESCRIPTION
       Change the IP address of a VM without logging on to the VM.

       Recent Version from 12/21/2013 works only with ip pools with /24 subnet IP pool range.
       Other IP pool ranges will be supported in a future version of this script.
       Example:
            IP pool range 10.0.0.1-10.0.0.254 with subnet mask 255.255.254.0 works!

            IP pool range 10.0.0.1-10.0.1.254 with subnet mask 255.255.254.0 does not work!
    .NOTES
        File Name : Set-SCVMIPAddress.ps1
        Author    : Daniel Neumann
        Requires  : PowerShell Version 3.0
                    System Center 2012 R2 Virtual Machine Manager PowerShell Cmdlets
    .LINK
        To provide feedback or for further assistance visit:
        http://www.danielstechblog.de  
    .EXAMPLE
       Set-SCVMIPAddress -VMMServer vmmserver.demo.local

       Call with the FQDN of the VMM Management Server.
    .EXAMPLE
       Set-SCVMIPAddress -VMMServer vmmserver

       Call with the name of the VMM Management Server.
#>
function Set-SCVMIPAddress
{
    param
    (
        [Parameter(Mandatory=$true, HelpMessage='VMM Server')]
        [String]
        $VMMServer
    )
    
    $VM=Get-SCVirtualMachine -VMMServer $VMMServer|Select-Object -Property Name|Out-Gridview -PassThru -Title 'Select Virtual Machine'
    $NIC=Get-SCVirtualNetworkAdapter -VMMServer $VMMServer -VM $VM.Name|Out-Gridview -PassThru -Title 'Select VMs vNIC'
    $VMNetwork=Get-SCVMNetwork -VMMServer $VMMServer|Out-Gridview -PassThru -Title 'Select VM Network'
    if($VMNetwork.VMSubnet){
        $VMSubnet=Get-SCVMSubnet -VMMServer $VMMServer -VMNetwork $VMNetwork|Out-GridView -PassThru -Title 'Select VM Subnet'
        $IPPool=Get-SCStaticIPAddressPool -VMMServer $VMMServer -VMSubnet $VMSubnet|Out-GridView -PassThru -Title 'Select IP address pool'
    }
    else{
        $LogicalNetworkDefinition=Get-SCLogicalNetworkDefinition -VMMServer $VMMServer -LogicalNetwork $VMNetwork.LogicalNetwork
        $IPPool=Get-SCStaticIPAddressPool -VMMServer $VMMServer -LogicalNetworkDefinition $LogicalNetworkDefinition |Out-GridView -PassThru -Title 'Select IP address pool'
    }
    $IPAddresses=Get-SCIPAddress -VMMServer $VMMServer -StaticIPAddressPool $IPPool -Assigned

    $FreeIPAddresses=@()
    if($IPPool.IPAddressRangeStart.Substring(0,$IPPool.IPAddressRangeStart.LastIndexOf(".")) -eq $IPPool.IPAddressRangeEnd.Substring(0,$IPPool.IPAddressRangeEnd.LastIndexOf("."))){
        $Start=[Int]$IPPool.IPAddressRangeStart.Substring($IPPool.IPAddressRangeStart.LastIndexOf(".")+1)
        $End=[Int]$IPPool.IPAddressRangeEnd.Substring($IPPool.IPAddressRangeEnd.LastIndexOf(".")+1)
        $ReservedIPs=$IPPool.IPAddressReservedSet.Split(",") -replace " ",""
        $Subnet=$IPPool.IPAddressRangeStart.Substring(0,$IPPool.IPAddressRangeStart.LastIndexOf("."))
        for($Start;$Start -le $End;$Start++){
            $Info = New-Object PSObject -Property @{
            Name="$Subnet.$Start"
            Info='Free'
            }
            $FreeIPAddresses += $Info
        }
        foreach($IPAddress in $IPAddresses){
            foreach($FreeIPAddress in $FreeIPAddresses){
                if($FreeIPAddress.Name -eq $IPAddress.Address){
                    $FreeIPAddress.Info='Assigned'
                }
            }
        }
        foreach($ReservedIP in $ReservedIPs){
            foreach($FreeIPAddress in $FreeIPAddresses){
                if($FreeIPAddress.Name -eq $ReservedIP){
                    $FreeIPAddress.Info='Reserved'
                }
            }
        }
        $ChoosenIPAddress=$FreeIPAddresses|Out-Gridview -PassThru -Title 'Select IP address'
        #Grant-SCIPAddress -VMMServer $VMMServer -StaticIPAddressPool $IPPool -GrantToObjectType VirtualNetworkAdapter -GrantToObjectID $NIC.ID -IPAddress $ChoosenIPAddress.Name|Set-SCIPAddress
        if($VMNetwork.VMSubnet){
            Set-SCVirtualNetworkAdapter -VMMServer $VMMServer -VirtualNetworkAdapter $NIC -VMNetwork $VMNetwork -VMSubnet $VMSubnet -IPv4AddressType Static -IPv4Addresses $ChoosenIPAddress.Name
        }
        else{
            Set-SCVirtualNetworkAdapter -VMMServer $VMMServer -VirtualNetworkAdapter $NIC -VMNetwork $VMNetwork -IPv4AddressType Static -IPv4Addresses $ChoosenIPAddress.Name
        }
    }
}